require "faraday"
require "json"

class BacklogJp::Client
  AuthenticationException = Class.new Exception
  APIException            = Class.new Exception

  def initialize config = {}
    config[:space]   ||= ENV["BACKLOG_SPACE"]
    config[:api_key] ||= ENV["BACKLOG_API_KEY"]

    unless config[:space] && config[:api_key]
      fail AuthenticationException, "`:space` and `:api_key` are required for authentication."
    end

    @space   = config[:space]
    @api_key = config[:api_key]
  end

  [:get, :post, :patch, :delete].each do |method|
    define_method method do |*args|
      api_path, params = *args
      params         ||= {}

      response = connection.send(method, "#{api_path}?apiKey=#{@api_key}", build_params(params))

      unless response.status.to_s.start_with? "2"
        fail APIException, JSON.parse(response.body, symbolize_names: true)
      end

      JSON.parse(response.body, symbolize_names: true)
    end
  end

  private

  def connection
    @connection ||= Faraday.new "https://#{@space}.backlog.jp/api/v2/"
  end

  def camelize string
    string.gsub(/_[a-z]/){|match| match.chars.last.upcase}
  end

  def build_params params
    hash = {}

    params.each do |key, value|
      hash.update camelize(key.to_s).intern => value
    end

    hash
  end
end
